@@ -83,6 +83,30 @@ select lineno, stmt_lineno, exec_stmts, source from plpgsql_profiler_function_tb
 
 drop function f1();
 
+-- test queryid retrieval
+create function f1()
+returns void as $$
+declare
+  t1 text = 't1';
+begin
+  insert into t1 values(10,20);
+  EXECUTE 'update ' ||  't1' || ' set a = 10';
+  EXECUTE 'delete from ' || t1;
+end;
+$$ language plpgsql;
+
+select plpgsql_profiler_reset_all();
+
+select plpgsql_profiler_install_fake_queryid_hook();
+
+select f1();
+
+select queryids, lineno, stmt_lineno, exec_stmts, source from plpgsql_profiler_function_tb('f1()');
+
+select plpgsql_profiler_remove_fake_queryid_hook();
+
+drop function f1();
+
 set plpgsql_check.profiler to off;
 
 create function f1()
@@ -4079,3 +4103,20 @@ set plpgsql_check.tracer to off;
 drop table tracer_tab cascade;
 drop function tracer_tab_trg_fx();
 drop function fxo(int, int, date, numeric);
+
+create or replace function foo_trg_func()
+returns trigger as $$
+begin
+  -- bad function, RETURN is missing
+end;
+$$ language plpgsql;
+
+create table foo(a int);
+
+create trigger foo_trg before insert for each row execute procedure foo_trg_func();
+
+-- should to print error
+select * from plpgsql_check_function('foo_trg_func', 'foo');
+
+drop table foo;
+drop function foo_trg_func();
