@@ -25,6 +25,14 @@
 
 #endif
 
+#if PG_VERSION_NUM >= 140000
+
+#include "nodes/nodeFuncs.h"
+#include "parser/parse_coerce.h"
+#include "utils/builtins.h"
+
+#endif
+
 #include "tcop/utility.h"
 #include "utils/lsyscache.h"
 
@@ -64,12 +72,34 @@ prepare_plan(PLpgSQL_checkstate *cstate,
 	{
 		MemoryContext old_cxt;
 
+#if PG_VERSION_NUM >= 140000
+
+		SPIPrepareOptions options;
+
+		memset(&options, 0, sizeof(options));
+		options.parserSetup = parser_setup ?
+			parser_setup : (ParserSetupHook) plpgsql_check__parser_setup_p;
+		options.parserSetupArg = arg ? arg : (void *) expr;
+		options.parseMode = expr->parseMode;
+		options.cursorOptions = cursorOptions;
+
+#endif
+
 		/*
 		 * The grammar can't conveniently set expr->func while building the parse
 		 * tree, so make sure it's set before parser hooks need it.
 		 */
 		expr->func = cstate->estate->func;
 
+#if PG_VERSION_NUM >= 140000
+
+		/*
+		 * Generate and save the plan
+		 */
+		plan = SPI_prepare_extended(expr->query, &options);
+
+#else
+
 		/*
 		 * Generate and save the plan
 		 */
@@ -78,6 +108,8 @@ prepare_plan(PLpgSQL_checkstate *cstate,
 								  arg ? arg : (void *) expr,
 								  cursorOptions);
 
+#endif
+
 		if (plan == NULL)
 		{
 			/* Some SPI errors deserve specific error messages */
@@ -875,7 +907,19 @@ plpgsql_check_expr_as_rvalue(PLpgSQL_checkstate *cstate, PLpgSQL_expr *expr,
 		 */
 		if (!type_is_rowtype(expected_typoid))
 			expand = false;
+
+#if PG_VERSION_NUM >= 140000
+
+		expr->target_param = targetdno;
 	}
+	else
+		expr->target_param = -1;
+
+#else
+
+	}
+
+#endif
 
 	oldowner = CurrentResourceOwner;
 	BeginInternalSubTransaction(NULL);
@@ -885,8 +929,113 @@ plpgsql_check_expr_as_rvalue(PLpgSQL_checkstate *cstate, PLpgSQL_expr *expr,
 	{
 		prepare_plan(cstate, expr, 0, NULL, NULL);
 		/* record all variables used by the query */
+
+#if PG_VERSION_NUM >= 140000
+
+		if (expr->target_param != -1)
+		{
+			int		target_dno = expr->target_param;
+			Node *node;
+			Oid		target_typoid = InvalidOid;
+			Oid		value_typoid = InvalidOid;
+
+			node = plpgsql_check_expr_get_node(cstate, expr, false);
+
+			if (bms_is_member(target_dno, expr->paramnos))
+			{
+				/* recheck if target_dno is really used on right side of assignment */
+				if (!plpgsql_check_vardno_is_used_for_reading(node, target_dno))
+				{
+					Bitmapset *paramnos;
+
+					/* create set without target_param */
+					paramnos = bms_copy(expr->paramnos);
+					paramnos = bms_del_member(paramnos, expr->target_param);
+					cstate->used_variables = bms_add_members(cstate->used_variables, paramnos);
+
+					bms_free(paramnos);
+				}
+				else
+					cstate->used_variables = bms_add_members(cstate->used_variables,
+															 expr->paramnos);
+			}
+			else
+				cstate->used_variables = bms_add_members(cstate->used_variables,
+														 expr->paramnos);
+
+			if (node && IsA(node, SubscriptingRef))
+				node = (Node *) ((SubscriptingRef *) node)->refassgnexpr;
+
+			/* check implicit coercion */
+			if (node && IsA(node, FuncExpr))
+			{
+				FuncExpr	   *fexpr = (FuncExpr *) node;
+
+				if (fexpr->funcformat == COERCE_IMPLICIT_CAST)
+				{
+					target_typoid = fexpr->funcresulttype;
+					value_typoid = exprType(linitial(fexpr->args));
+				}
+			}
+			else if (node && IsA(node, CoerceViaIO))
+			{
+				CoerceViaIO	   *cexpr = (CoerceViaIO *) node;
+
+				if (cexpr->coerceformat == COERCE_IMPLICIT_CAST)
+				{
+					target_typoid = cexpr->resulttype;
+					value_typoid = exprType((Node *) cexpr->arg);
+				}
+			}
+
+			if (target_typoid != value_typoid)
+			{
+				StringInfoData	str;
+
+				initStringInfo(&str);
+				appendStringInfo(&str, "cast \"%s\" value to \"%s\" type",
+											format_type_be(value_typoid),
+											format_type_be(target_typoid));
+
+				/* accent warning when cast is without supported explicit casting */
+				if (!can_coerce_type(1, &value_typoid, &target_typoid, COERCION_EXPLICIT))
+					plpgsql_check_put_error(cstate,
+								  ERRCODE_DATATYPE_MISMATCH, 0,
+								  "target type is different type than source type",
+								  str.data,
+								  "There are no possible explicit coercion between those types, possibly bug!",
+								  PLPGSQL_CHECK_WARNING_OTHERS,
+								  0, NULL, NULL);
+				else if (!can_coerce_type(1, &value_typoid, &target_typoid, COERCION_ASSIGNMENT))
+					plpgsql_check_put_error(cstate,
+								  ERRCODE_DATATYPE_MISMATCH, 0,
+								  "target type is different type than source type",
+								  str.data,
+								  "The input expression type does not have an assignment cast to the target type.",
+								  PLPGSQL_CHECK_WARNING_OTHERS,
+								  0, NULL, NULL);
+				else
+					plpgsql_check_put_error(cstate,
+								  ERRCODE_DATATYPE_MISMATCH, 0,
+								  "target type is different type than source type",
+								  str.data,
+								  "Hidden casting can be a performance issue.",
+								  PLPGSQL_CHECK_WARNING_PERFORMANCE,
+								  0, NULL, NULL);
+
+				pfree(str.data);
+			}
+		}
+		else
+			cstate->used_variables = bms_add_members(cstate->used_variables,
+													 expr->paramnos);
+
+#else
+
 		cstate->used_variables = bms_add_members(cstate->used_variables, expr->paramnos);
 
+#endif
+
 		/*
 		 * there is a possibility to call a plpgsql_pragma like default for some aux
 		 * variable. When we detect this case, then we mark target variable as used
