@@ -7,6 +7,15 @@
 #include "access/tupdesc.h"
 #include "storage/ipc.h"
 
+#if PG_VERSION_NUM >= 110000
+typedef uint64 pc_queryid;
+#define NOQUERYID				(UINT64CONST(0))
+#else
+typedef uint32 pc_queryid;
+#define NOQUERYID	(0)
+#endif
+
+
 enum
 {
 	PLPGSQL_CHECK_ERROR,
@@ -168,14 +177,15 @@ extern void plpgsql_check_put_error(PLpgSQL_checkstate *cstate, int sqlerrcode,
 	const char *message, const char *detail, const char *hint, int level, int position, const char *query, const char *context);
 extern void plpgsql_check_put_error_edata(PLpgSQL_checkstate *cstate, ErrorData *edata);
 extern void plpgsql_check_put_dependency(plpgsql_check_result_info *ri, char *type, Oid oid, char *schema, char *name, char *params);
-extern void plpgsql_check_put_profile(plpgsql_check_result_info *ri, int lineno, int stmt_lineno,
+extern void plpgsql_check_put_profile(plpgsql_check_result_info *ri, Datum queryids_array, int lineno, int stmt_lineno,
 	int cmds_on_row, int exec_count, int64 us_total, Datum max_time_array, Datum processed_rows_array, char *source_row);
-extern void plpgsql_check_put_profile_statement(plpgsql_check_result_info *ri, int stmtid, int parent_stmtid, const char *parent_note, int block_num, int lineno,
+extern void plpgsql_check_put_profile_statement(plpgsql_check_result_info *ri, pc_queryid queryid, int stmtid, int parent_stmtid, const char *parent_note, int block_num, int lineno,
 	int64 exec_stmts, double total_time, double max_time, int64 processed_rows, char *stmtname);
 
 /*
  * function from catalog.c
  */
+extern bool plpgsql_check_is_eventtriggeroid(Oid typoid);
 extern void plpgsql_check_get_function_info(HeapTuple procTuple, Oid *rettype, char *volatility, PLpgSQL_trigtype *trigtype, bool *is_procedure);
 extern void plpgsql_check_precheck_conditions(plpgsql_check_info *cinfo);
 extern char *plpgsql_check_get_src(HeapTuple procTuple);
@@ -220,6 +230,7 @@ extern void plpgsql_check_funcexpr(PLpgSQL_checkstate *cstate, Query *query, cha
 extern bool plpgsql_check_is_sql_injection_vulnerable(PLpgSQL_checkstate *cstate, PLpgSQL_expr *expr, Node *node, int *location);
 extern bool plpgsql_check_contain_volatile_functions(Node *clause, PLpgSQL_checkstate *cstate);
 extern bool plpgsql_check_contain_mutable_functions(Node *clause, PLpgSQL_checkstate *cstate);
+extern bool plpgsql_check_vardno_is_used_for_reading(Node *node, int dno);
 
 /*
  * functions from check_expr.c
@@ -289,6 +300,7 @@ extern Oid plpgsql_check_parse_name_or_signature(char *name_or_signature);
  * functions from profiler.c
  */
 extern bool plpgsql_check_profiler;
+extern int plpgsql_check_profiler_max_shared_chunks;
 
 extern Size plpgsql_check_shmem_size(void);
 extern void plpgsql_check_profiler_init_hash_tables(void);
@@ -411,15 +423,11 @@ extern plpgsql_check__recognize_err_condition_t plpgsql_check__recognize_err_con
 #define recvar_tuple(rec)		(rec->erh ? expanded_record_get_tuple(rec->erh) : NULL)
 #define recvar_tupdesc(rec)		(rec->erh ? expanded_record_fetch_tupdesc(rec->erh) : NULL)
 
-#define is_procedure(estate)		((estate)->func && (estate)->func->fn_rettype == InvalidOid)
-
 #else
 
 #define recvar_tuple(rec)		(rec->tup)
 #define recvar_tupdesc(rec)		(rec->tupdesc)
 
-#define is_procedure(estate)	(false)
-
 #endif
 
 #if PG_VERSION_NUM >= 100000
